function D = d2kgf(v,t,m,r,p,k);

%________________________________________________________________________________ 
% KGF.M
% Arguments: v (argument, 2 X 1), t (gridpoint), m (base period), r (possible
% claim amounts, n X 1), p (probabilities of settlements at different periods,
% n X 1), k (number of claims)
% This function computes the cumulant generating function of the insurer's 
% total claim amount with the M-P decomposition  
%________________________________________________________________________________ 

n = length(r); % number of different claims
tol = 10^(-15); 

D2_11 = 0; D2_12 = 0; D2_22 = 0;
for j = 1:n,
  s = s(j,v,t,m,r,p,k,tol);
  s_1 = ds1(j,v,t,m,r,p,k,tol); 
  s_2 = ds2(j,v,t,m,r,p,k,tol);  
  s_11 = ds11(j,v,t,m,r,p,k,tol);
  s_22 = ds22(j,v,t,m,r,p,k,tol);
  s_12 = ds12(j,v,t,m,r,p,k,tol);
  D2_11 = D2_11 + (s_11*(1+s)-s_1^2)/(1+s)^2;
  D2_22 = D2_22 + (s_22*(1+s)-s_2^2)/(1+s)^2;
  D2_12 = D2_12 + (s_12*(1+s)-s_1*s_2)/(1+s)^2;
end;

D = [D2_11 D2_12; D2_12 D2_22];       % second order derivatives of K

%________________________________________________________________________________
function g = s(j,v,t,m,r,p,k,tol);
g = 0;  
l = 1;
summand = 1;
while abs(summand)>tol,
  summand = exp((v(1)*r(j)*((j<=m)-t)+v(2)+log(k*p(j)))*l)/gamma(l+1);  
  g = g + summand; 
  l = l+1;
end;

function g = ds1(j,v,t,m,r,p,k,tol);
g = 0;     
l = 1;
summand = 1;
while abs(summand)>tol,
  summand = r(j)*((j<=m)-t)*exp((v(1)*r(j)*((j<=m)-t)+v(2)+log(k*p(j)))*l)/gamma(l);    
  g = g + summand;     
  l = l+1;
end;

function g = ds2(j,v,t,m,r,p,k,tol);
g = 0;     
l = 1;
summand = 1;
while abs(summand)>tol,
  summand = exp((v(1)*r(j)*((j<=m)-t)+v(2)+log(k*p(j)))*l)/gamma(l);    
  g = g + summand;     
  l = l+1;
end;

function g = ds11(j,v,t,m,r,p,k,tol);
g = 0;
l = 1;
summand = 1;
while abs(summand)>tol,
  summand = l*(r(j)*((j<=m)-t))^2*exp((v(1)*r(j)*((j<=m)-t)+v(2)+log(k*p(j)))*l)/gamma(l);  
  g = g + summand;
  l = l+1;
end;

function g = ds22(j,v,t,m,r,p,k,tol);
g = 0;     
l = 1;
summand = 1;
while abs(summand)>tol,
  summand = l*exp((v(1)*r(j)*((j<=m)-t)+v(2)+log(k*p(j)))*l)/gamma(l);    
  g = g + summand;     
  l = l+1;
end;

function g = ds12(j,v,t,m,r,p,k,tol);
g = 0;
l = 1;
summand = 1;
while abs(summand)>tol,
  summand = l*r(j)*((j<=m)-t)*exp((v(1)*r(j)*((j<=m)-t)+v(2)+log(k*p(j)))*l)/gamma(l);  
  g = g + summand;
  l = l+1;
end;

