function prob = lr(v,t,m,r,p,k);

%______________________________________________________________________________
% LR.M
% Arguments: v (saddlepoint 2X1), t (gridpoint), m (number of balls of each 
%            color, vector), k (number of draws)
% This function computes the Lugannani and Rice saddlepoint approximation for
% the coloration entropy estimator under sampling without replacement and the 
% MH-B decomposition
%______________________________________________________________________________

D = d2kgf(v,t,m,r,p,k);
D0 = d2kgf([0 0],t,m,r,p,k); 

s = v(1)*sqrt( det(D)/k );
K = kgf(v,t,m,r,p,k);
if K > 0,
	prob = NaN;
	return 
end;
r = sign(v(1))*sqrt(-2*K);
phi = 1/sqrt(2*pi)*exp(-r^2/2);
prob = normal(r) - phi*(1/s-1/r);
