function D = d2kgf(v,t,p,k);

%________________________________________________________________________________ 
% D2KGF.M
% Arguments: v (argument, 2 X 1), t (gridpoint), p (multinonial probabilities), 
%            k (total number balls)
% This function computes the second order derivatives of the cumulant 
% generating function of the coloration entropy estimator under sampling with 
% replacement and the M-P decomposition 
%________________________________________________________________________________ 

n = length(p); % number of different colors

tol = 10^(-15);
D2_11 = 0; D2_12 = 0; D2_22 = 0;
for j = 1:n,
  Sf = 0; Sf1 = 0; Sf2 = 0; Sf11 = 0; Sf22 = 0; Sf12 = 0; 
  l = 1;
  level = 1;
  while level > 10^(-15),
    summand   = f(v(1),v(2),(k*p(j))^l/gamma(l+1),l);
    summand1  = d1_f(v(1),v(2),(k*p(j))^l/gamma(l+1),l);
    summand2  = d2_f(v(1),v(2),(k*p(j))^l/gamma(l+1),l);
    summand11 = d11_f(v(1),v(2),(k*p(j))^l/gamma(l+1),l);
    summand22 = d22_f(v(1),v(2),(k*p(j))^l/gamma(l+1),l);
    summand12 = d12_f(v(1),v(2),(k*p(j))^l/gamma(l+1),l);
    Sf = Sf + summand; 
    Sf1 = Sf1 + summand1; 
    Sf2 = Sf2 + summand2;
    Sf11 = Sf11 + summand11; 
    Sf22 = Sf22 + summand22; 
    Sf12 = Sf12 + summand12; 
    l = l+1; 
    level = max([summand summand1 summand2 summand11 summand22 summand12]);
  end;
  D2_11 = D2_11 + (Sf11*(1+Sf)-Sf1^2)/(1+Sf)^2; 
  D2_22 = D2_22 + (Sf22*(1+Sf)-Sf2^2)/(1+Sf)^2;
  D2_12 = D2_12 + (Sf12*(1+Sf)-Sf1*Sf2)/(1+Sf)^2;
end;

D = [D2_11 D2_12; D2_12 D2_22];

%________________________________________________________________________________
function g = f(v1,v2,a,l);
g = a*(l^(-v1)*exp(v2))^l;

function dv1_f = d1_f(v1,v2,a,l);
dv1_f = -a*l*log(l)*l^(-v1*l)*exp(l*v2);

function dv2_f = d2_f(v1,v2,a,l);
dv2_f = a*l*l^(-v1*l)*exp(l*v2);

function dv1v1_f = d11_f(v1,v2,a,l);
dv1v1_f = a*l^2*(log(l))^2*l^(-v1*l)*exp(l*v2);

function dv2v2_f = d22_f(v1,v2,a,l);
dv2v2_f = a*l^(-l*v1+2)*exp(l*v2);  

function dv1v2_f = d12_f(v1,v2,a,l);
dv1v2_f = -a*l^2*log(l)*l^(-v1*l)*exp(l*v2);  

