function D = d2kgf(v,t,m,k);

%________________________________________________________________________________ 
% D2KGF.M
% Arguments: v (argument, 2 X 1), t (gridpoint), m (number of balls of each 
% 	     color, vector), k (number of draws)  
% This function computes the first and second order derivatives of the cumulant 
% generating function of the coloration entropy estimator under sampling without 
% replacement and the MH-B decomposition 
%________________________________________________________________________________ 

n = length(m); % number of different colors
z = sum(m);

D1_1 = 0; D1_2 = 0;
D2_11 = 0; D2_12 = 0; D2_22 = 0;
for j = 1:n,
  Sf = 0; Sf1 = 0; Sf2 = 0; Sf11 = 0; Sf22 = 0; Sf12 = 0; 
  for l = 1:m(j),
    Sf = Sf +         f(v(1),v(2),nchoosek(m(j),l)*(k/(z-k))^l,l);
    Sf1 = Sf1 +    d1_f(v(1),v(2),nchoosek(m(j),l)*(k/(z-k))^l,l);
    Sf2 = Sf2 +    d2_f(v(1),v(2),nchoosek(m(j),l)*(k/(z-k))^l,l);
    Sf11 = Sf11 + d11_f(v(1),v(2),nchoosek(m(j),l)*(k/(z-k))^l,l);
    Sf22 = Sf22 + d22_f(v(1),v(2),nchoosek(m(j),l)*(k/(z-k))^l,l);
    Sf12 = Sf12 + d12_f(v(1),v(2),nchoosek(m(j),l)*(k/(z-k))^l,l);
  end;
  D1_1 = D1_1 + Sf1/(1+Sf);
  D1_2 = D1_2 + Sf2/(1+Sf);
  D2_11 = D2_11 + (Sf11*(1+Sf)-Sf1^2)/(1+Sf)^2; 
  D2_22 = D2_22 + (Sf22*(1+Sf)-Sf2^2)/(1+Sf)^2;
  D2_12 = D2_12 + (Sf12*(1+Sf)-Sf1*Sf2)/(1+Sf)^2;
end;

d1K = [-k*(t-log(k)) + D1_1; -k + D1_2]; % first order derivatives of K
d2K = [D2_11 D2_12; D2_12 D2_22];	% second order derivatives of K
D = [d1K d2K]; 

%________________________________________________________________________________
function g = f(v1,v2,a,l);
g = a*(l^(-v1)*exp(v2))^l;

function dv1_f = d1_f(v1,v2,a,l);
dv1_f = -a*l*log(l)*l^(-v1*l)*exp(l*v2);

function dv2_f = d2_f(v1,v2,a,l);
dv2_f = a*l*l^(-v1*l)*exp(l*v2);

function dv1v1_f = d11_f(v1,v2,a,l);
dv1v1_f = a*l^2*(log(l))^2*l^(-v1*l)*exp(l*v2);

function dv2v2_f = d22_f(v1,v2,a,l);
dv2v2_f = a*l^(-l*v1+2)*exp(l*v2);  

function dv1v2_f = d12_f(v1,v2,a,l);
dv1v2_f = -a*l^2*log(l)*l^(-v1*l)*exp(l*v2);  

